"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.themeable = themeable;
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _hoistNonReactStatics = _interopRequireDefault(require("hoist-non-react-statics"));

var _invariant = _interopRequireDefault(require("invariant"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * @typedef {Object.<string, TReactCSSThemrTheme>} TReactCSSThemrTheme
 */

/**
 * @typedef {{}} TReactCSSThemrOptions
 * @property {String|Boolean} [composeTheme=COMPOSE_DEEPLY]
 */
const COMPOSE_DEEPLY = 'deeply';
const COMPOSE_SOFTLY = 'softly';
const DONT_COMPOSE = false;
const DEFAULT_OPTIONS = {
  composeTheme: COMPOSE_DEEPLY,
  mapThemrProps: defaultMapThemrProps
};
const THEMR_CONFIG = typeof Symbol !== 'undefined' ? Symbol('THEMR_CONFIG') : '__REACT_CSS_THEMR_CONFIG__';
/**
 * Themr decorator
 * @param {String|Number|Symbol} componentName - Component name
 * @param {TReactCSSThemrTheme} [localTheme] - Base theme
 * @param {{}} [options] - Themr options
 * @returns {function(ThemedComponent:Function):Function} - ThemedComponent
 */

const themr = (componentName, localTheme, options = {}) => ThemedComponent => {
  const _DEFAULT_OPTIONS$opti = _objectSpread(_objectSpread({}, DEFAULT_OPTIONS), options),
        optionComposeTheme = _DEFAULT_OPTIONS$opti.composeTheme,
        optionMapThemrProps = _DEFAULT_OPTIONS$opti.mapThemrProps;

  validateComposeOption(optionComposeTheme);
  let config = ThemedComponent[THEMR_CONFIG];

  if (config && config.componentName === componentName) {
    config.localTheme = themeable(config.localTheme, localTheme);
    return ThemedComponent;
  }

  config = {
    componentName,
    localTheme
  };
  /**
   * @property {{wrappedInstance: *}} refs
   */

  class Themed extends _react.Component {
    constructor(...args) {
      super(...args);
      this.theme_ = this.calcTheme(this.props);
    }

    getWrappedInstance() {
      (0, _invariant.default)(true, 'DEPRECATED: To access the wrapped instance, you have to pass ' + '{ innerRef: fn } and retrieve with a callback ref style.');
      return this.refs.wrappedInstance;
    }

    getNamespacedTheme(props) {
      const themeNamespace = props.themeNamespace,
            theme = props.theme;
      if (!themeNamespace) return theme;

      if (themeNamespace && !theme) {
        throw new Error('Invalid themeNamespace use in friendsofreactjs/react-css-themr. ' + 'themeNamespace prop should be used only with theme prop.');
      }

      return Object.keys(theme).filter(key => key.startsWith(themeNamespace)).reduce((result, key) => _objectSpread(_objectSpread({}, result), {}, {
        [removeNamespace(key, themeNamespace)]: theme[key]
      }), {});
    }

    getThemeNotComposed(props) {
      const theme = props.theme;
      if (theme) return this.getNamespacedTheme(props);
      if (config.localTheme) return config.localTheme;
      return this.getContextTheme();
    }

    getContextTheme() {
      return this.context.themr ? this.context.themr.theme[config.componentName] : {};
    }

    getTheme(props) {
      const composeTheme = props.composeTheme;
      return composeTheme === COMPOSE_SOFTLY ? _objectSpread(_objectSpread(_objectSpread({}, this.getContextTheme()), config.localTheme), this.getNamespacedTheme(props)) : themeable(themeable(this.getContextTheme(), config.localTheme), this.getNamespacedTheme(props));
    }

    calcTheme(props) {
      const composeTheme = props.composeTheme;
      return composeTheme ? this.getTheme(props) : this.getThemeNotComposed(props);
    }

    shouldComponentUpdate(nextProps) {
      const _this$props = this.props,
            composeTheme = _this$props.composeTheme,
            theme = _this$props.theme,
            themeNamespace = _this$props.themeNamespace;

      if (composeTheme !== nextProps.composeTheme || theme !== nextProps.theme || themeNamespace !== nextProps.themeNamespace) {
        this.theme_ = this.calcTheme(nextProps);
      }

      return true;
    }

    render() {
      return /*#__PURE__*/_react.default.createElement(ThemedComponent, this.props.mapThemrProps(this.props, this.theme_));
    }

  }

  _defineProperty(Themed, "displayName", "Themed".concat(ThemedComponent.displayName || ThemedComponent.name || 'Component'));

  _defineProperty(Themed, "contextTypes", {
    themr: _propTypes.default.object
  });

  _defineProperty(Themed, "propTypes", _objectSpread(_objectSpread({}, ThemedComponent.propTypes), {}, {
    composeTheme: _propTypes.default.oneOf([COMPOSE_DEEPLY, COMPOSE_SOFTLY, DONT_COMPOSE]),
    innerRef: _propTypes.default.oneOfType([_propTypes.default.func, _propTypes.default.object]),
    theme: _propTypes.default.object,
    themeNamespace: _propTypes.default.string,
    mapThemrProps: _propTypes.default.func
  }));

  _defineProperty(Themed, "defaultProps", _objectSpread(_objectSpread({}, ThemedComponent.defaultProps), {}, {
    composeTheme: optionComposeTheme,
    mapThemrProps: optionMapThemrProps
  }));

  Themed[THEMR_CONFIG] = config;
  return (0, _hoistNonReactStatics.default)(Themed, ThemedComponent);
};

var _default = themr;
/**
 * Merges passed themes by concatenating string keys and processing nested themes
 *
 * @param {...TReactCSSThemrTheme} themes - Themes
 * @returns {TReactCSSThemrTheme} - Resulting theme
 */

exports.default = _default;

function themeable(...themes) {
  return themes.reduce((acc, theme) => merge(acc, theme), {});
}
/**
 * @param {TReactCSSThemrTheme} [original] - Original theme
 * @param {TReactCSSThemrTheme} [mixin] - Mixin theme
 * @returns {TReactCSSThemrTheme} - resulting theme
 */


function merge(original = {}, mixin = {}) {
  //make a copy to avoid mutations of nested objects
  //also strip all functions injected by isomorphic-style-loader
  const result = Object.keys(original).reduce((acc, key) => {
    const value = original[key];

    if (typeof value !== 'function') {
      acc[key] = value;
    }

    return acc;
  }, {}); //traverse mixin keys and merge them to resulting theme

  Object.keys(mixin).forEach(key => {
    //there's no need to set any defaults here
    const originalValue = result[key];
    const mixinValue = mixin[key];

    switch (typeof mixinValue) {
      case 'object':
        {
          //possibly nested theme object
          switch (typeof originalValue) {
            case 'object':
              {
                //exactly nested theme object - go recursive
                result[key] = merge(originalValue, mixinValue);
                break;
              }

            case 'undefined':
              {
                //original does not contain this nested key - just take it as is
                result[key] = mixinValue;
                break;
              }

            default:
              {
                //can't merge an object with a non-object
                throw new Error("You are merging object ".concat(key, " with a non-object ").concat(originalValue));
              }
          }

          break;
        }

      case 'undefined': //fallthrough - handles accidentally unset values which may come from props

      case 'function':
        {
          //this handles issue when isomorphic-style-loader addes helper functions to css-module
          break; //just skip
        }

      default:
        {
          //plain values
          switch (typeof originalValue) {
            case 'object':
              {
                //can't merge a non-object with an object
                throw new Error("You are merging non-object ".concat(mixinValue, " with an object ").concat(key, ", (can occur when using empty or :global only base theme stylesheet)"));
              }

            case 'undefined':
              {
                //mixin key is new to original theme - take it as is
                result[key] = mixinValue;
                break;
              }

            case 'function':
              {
                //this handles issue when isomorphic-style-loader addes helper functions to css-module
                break; //just skip
              }

            default:
              {
                //finally we can merge
                result[key] = originalValue.split(' ').concat(mixinValue.split(' ')).filter((item, pos, self) => self.indexOf(item) === pos && item !== '').join(' ');
                break;
              }
          }

          break;
        }
    }
  });
  return result;
}
/**
 * Validates compose option
 *
 * @param {String|Boolean} composeTheme - Compose them option
 * @throws
 * @returns {undefined}
 */


function validateComposeOption(composeTheme) {
  if ([COMPOSE_DEEPLY, COMPOSE_SOFTLY, DONT_COMPOSE].indexOf(composeTheme) === -1) {
    throw new Error("Invalid composeTheme option for friendsofreactjs/react-css-themr. Valid composition options are ".concat(COMPOSE_DEEPLY, ", ").concat(COMPOSE_SOFTLY, " and ").concat(DONT_COMPOSE, ". The given option was ").concat(composeTheme));
  }
}
/**
 * Removes namespace from key
 *
 * @param {String} key - Key
 * @param {String} themeNamespace - Theme namespace
 * @returns {String} - Key
 */


function removeNamespace(key, themeNamespace) {
  const capitalized = key.substr(themeNamespace.length);
  return capitalized.slice(0, 1).toLowerCase() + capitalized.slice(1);
}
/**
 * Maps props and theme to an object that will be used to pass down props to the
 * decorated component.
 *
 * @param {Object} ownProps - All props given to the decorated component
 * @param {Object} theme - Calculated then that should be passed down
 * @returns {Object} - Props that will be passed down to the decorated component
 */


function defaultMapThemrProps(ownProps, theme) {
  const composeTheme = ownProps.composeTheme,
        innerRef = ownProps.innerRef,
        themeNamespace = ownProps.themeNamespace,
        mapThemrProps = ownProps.mapThemrProps,
        rest = _objectWithoutProperties(ownProps, ["composeTheme", "innerRef", "themeNamespace", "mapThemrProps"]);

  return _objectSpread(_objectSpread({}, rest), {}, {
    ref: innerRef,
    theme
  });
}